@@ -105,7 +105,11 @@ check_gold gold-67-ln.txt           tests/input-{6,7}.txt --cols=80 --line-numbe
 check_gold gold-67-u3.txt           tests/input-{6,7}.txt --cols=80 -U 3
 check_gold gold-tabs-default.txt    tests/input-{8,9}.txt --cols=80
 check_gold gold-tabs-4.txt          tests/input-{8,9}.txt --cols=80 --tabsize=4
-
+check_gold gold-file-not-found.txt  tests/input-4.txt nonexistent_file
+check_gold gold-strip-cr-off.txt    tests/input-4.txt tests/input-4-cr.txt --cols=80
+check_gold gold-strip-cr-on.txt     tests/input-4.txt tests/input-4-cr.txt --cols=80 --strip-trailing-cr
+check_gold gold-no-cr-indent        tests/input-4-cr.txt tests/input-4-partial-cr.txt --cols=80 
+check_gold gold-hide-cr-if-dos      tests/input-4-cr.txt tests/input-5-cr.txt --cols=80
 
 if [ ! -z "$INSTALLED" ]; then
   VERSION=$(icdiff --version | awk '{print $NF}')
@@ -117,6 +121,20 @@ if [ "$VERSION" != $(head -n 1 ChangeLog) ]; then
   fail
 fi
 
+if ! command -v 'flake8' >/dev/null 2>&1; then
+  echo 'Could not find flake8. Ensure flake8 is installed and on your $PATH.'
+  if [ -z "$VIRTUAL_ENV" ]; then
+    echo 'It appears you have have forgotten to activate your virtualenv.'
+  fi
+  echo 'See README.md for details on setting up your environment.'
+  fail
+fi
+
+echo 'Running flake8 linter...'
+if ! flake8 icdiff; then
+  fail
+fi
+
 if ! $REGOLD; then
   echo PASS
 fi
